import pytest
import rpm
from rpmlint.checks.AtomicUpdateCheck import AtomicUpdateCheck
from rpmlint.filter import Filter

from Testing import CONFIG, get_tested_mock_package


@pytest.fixture(scope='function', autouse=True)
def atomiccheck():
    CONFIG.info = True
    CONFIG.configuration['AtomicCheckGhosts'] = True
    output = Filter(CONFIG)
    test = AtomicUpdateCheck(CONFIG, output)
    yield output, test


@pytest.fixture
def output(atomiccheck):
    output, _test = atomiccheck
    yield output


@pytest.fixture
def test(atomiccheck):
    _output, test = atomiccheck
    yield test


@pytest.mark.parametrize('package', [
    get_tested_mock_package(files=('/var/lib/pipewire',)),
    get_tested_mock_package(files=('/opt/bin/test',)),
    get_tested_mock_package(files=('/usr/local/bin/test',)),
    get_tested_mock_package(files=('/boot/efi/test',)),
])
def test_not_atomic(package, output, test):
    test.check(package)
    out = output.print_results(output.results)
    assert 'E: dir-or-file-outside-snapshot' in out


@pytest.mark.parametrize('package', [
    get_tested_mock_package(files=('/etc/custom.config',)),
    get_tested_mock_package(files=('/usr/lib64/libc.so',)),
    get_tested_mock_package(files=('/usr/etc/nfs.conf',)),
    get_tested_mock_package(files=('/bin/test',)),
    get_tested_mock_package(files=('/sbin/test',)),
    get_tested_mock_package(files=('/lib/libc.so',)),
    get_tested_mock_package(files=('/lib64/libc.so',)),
    get_tested_mock_package(files=('/boot/grub2/grub.cfg',)),
])
def test_atomic(package, output, test):
    test.check(package)
    out = output.print_results(output.results)
    assert 'E: dir-or-file-outside-snapshot' not in out
    assert 'W: ghost-outside-snapshot' not in out


@pytest.mark.parametrize('package', [
    get_tested_mock_package(files={
        '/var/lib/pipewire/ghost_file': {'metadata': {'flags': rpm.RPMFILE_GHOST}},
    }),
])
def test_not_atomic_ghost(package, output, test):
    test.check(package)
    out = output.print_results(output.results)
    assert 'W: ghost-outside-snapshot' in out



from rpmlint.checks.AbstractCheck import AbstractCheck


class AtomicUpdateCheck(AbstractCheck):

    """
    Requirements for atomic updates:
        * All files must be stored inside the snapshot, which is in our case /etc and /usr, not /var,
          /opt, /srv, /usr/local or anything else.
        * (Re)starting daemons is not possible.
        * Modifying files outside of /usr and /etc is not possible.
        * Modifications outside the snapshot have to be done via systemd-tmpfiles and systemd services.
    This check currently only implements checking for files at illegal paths.
    """

    def __init__(self, config, output):
        super().__init__(config, output)
        self.check_ghosts = self.config.configuration['AtomicCheckGhosts']
        self.allowed_dirs = self.config.configuration['AtomicAllowedDirs']
        self.disallowed_subdirs = self.config.configuration['AtomicDisallowedSubdirs']

    def check(self, pkg):
        if pkg.is_source:
            return

        # Check for files stored outside the snapshot
        self._check_paths(pkg, self.check_ghosts)

    def _check_paths(self, pkg, check_ghosts=False):
        for file in pkg.files.keys():
            if file in pkg.ghost_files:
                continue  # Ghosts are only handled if explicitly desired
            if not (self._check_single_path(file)):
                self.output.add_info('E', pkg, 'dir-or-file-outside-snapshot', file)
        if check_ghosts:
            for ghost in pkg.ghost_files:
                if not (self._check_single_path(ghost)):
                    self.output.add_info('W', pkg, 'ghost-outside-snapshot', ghost)

    def _check_single_path(self, file):
        return (
            file.startswith(tuple(self.allowed_dirs)) and
            not file.startswith(tuple(self.disallowed_subdirs))
        )
